<?php
/*
  Plugin Name: NextGen Gallery Powertags
  Plugin URI: http://www.mauromascia.com/portfolio/wordpress-nextgen-gallery-powertags
  Description: Extends NextGen Gallery to simplify the image filtering, providing a simple way to filter the gallery images with their own tags.
  Version: 1.1
  Author: Mauro Mascia (baba_mmx)
  Author URI: http://www.mauromascia.com
  License: GPLv2
  Support: info@mauromascia.com

  Copyright 2012  Mauro Mascia (email: info@mauromascia.com)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2, as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
if (!is_plugin_active("nextgen-gallery/nggallery.php")) {
  echo "You must install NextGen Gallery before.";
  return;
}

/**
 * Creates the [nggpowertags] shortcode
 */
function nggpowertags_shortcode($atts) {
  extract(shortcode_atts(array(
              'gallery' => null,
              'tagmenu_format' => 'sep', // 'sep' OR 'list'
              'tagmenu_sep' => ',', // any char or html stuff to separate the tags
              'all_word' => 'all', // tell it in your language, uppercase or what you want
              'template' => '',
              'exclude' => ''
                  ), $atts));
  
  list($menu, $taglist) = nggpowertags_GetGalleryTags($gallery, $tagmenu_format, $tagmenu_sep, $all_word, $exclude);
  
  return nggpowertags_ShowGalleryTags($gallery, $template, $menu, $taglist);
}

add_shortcode('nggpowertags', 'nggpowertags_shortcode');

/**
 * Shows all images or the filtered ones
 */
function nggpowertags_ShowGalleryTags($gallery, $template, $menu, $taglist) {
  $the_tag = get_query_var('gallerytag'); // $_GET from wp_query
  
  // Set images for a certain tag or for all tags of the selected gallery
  $taglist = !empty($the_tag) && $the_tag != "all" ? $the_tag . "," : $taglist;

  // get related images
  //$picturelist = nggTags::find_images_for_tags($taglist, 'ASC');
  $picturelist = nggpowertags_find_images_for_tags($taglist, 'ASC', $gallery);

  if (empty($picturelist)) {
    echo "taglist is empty";
    return;
  }
  
  echo $menu; // show tag menu
  
  // show gallery
  if (is_array($picturelist))
    $out = nggCreateGallery($picturelist, $gallery, $template);

  $out = apply_filters('ngg_show_gallery_tags_content', $out, $taglist);
  return $out;
}

/**
 * Get images corresponding to a list of tags
 * 
 * This is a modification of the original ngg function find_images_for_tags
 * (on nggfunctions.php) to allows the filtering by the gallery ID
 */
function nggpowertags_find_images_for_tags($taglist, $mode = "ASC", $galleryID = null) {
  global $wpdb;

  // extract it into a array
  $taglist = explode(",", $taglist);

  if (!is_array($taglist))
    $taglist = array($taglist);

  $taglist = array_map('trim', $taglist);
  $new_slugarray = array_map('sanitize_title', $taglist);
  $sluglist = "'" . implode("', '", $new_slugarray) . "'";

  //Treat % as a litteral in the database, for unicode support
  $sluglist = str_replace("%", "%%", $sluglist);

  // first get all $term_ids with this tag		
  $term_ids = $wpdb->get_col(
    $wpdb->prepare(
      "SELECT term_id FROM $wpdb->terms WHERE slug IN ($sluglist) ORDER BY term_id ASC "
    )
  );
  
  /*
   * this function stores all pictures with the tags passed ...
   * but we want the ones only from our gallery. So I have to re-write
   * also find_images_in_list
   */
  $picids = get_objects_in_term($term_ids, 'ngg_tag');

  //Now lookup in the database
  if ($mode == 'RAND') {
    //$pictures = nggdb::find_images_in_list($picids, true, 'RAND');
    $pictures = nggpowertags_find_images_in_list($picids, true, 'RAND', $galleryID);
  }
  else {
    //$pictures = nggdb::find_images_in_list($picids, true, 'ASC');
    $pictures = nggpowertags_find_images_in_list($picids, true, 'ASC', $galleryID);
  }
  
  return $pictures;
}

/**
 * Get images given a list of IDs 
 * 
 * This is a modification of the original ngg function find_images_in_list
 * (on ngg-db.php) to allows the filtering by the gallery ID
 */
function nggpowertags_find_images_in_list($pids, $exclude = false, $order = 'ASC', $galleryID = null) {
  global $wpdb;

  $result = array();

  // Check for the exclude setting
  $exclude_clause = ($exclude) ? ' AND t.exclude <> 1 ' : '';

  // Check for the exclude setting
  $order_clause = ($order == 'RAND') ? 'ORDER BY rand() ' : ' ORDER BY t.pid ASC';

  if (is_array($pids)) {
    $id_list = "'" . implode("', '", $pids) . "'";

    // Save Query database
    $images = $wpdb->get_results(
            "SELECT t.*, tt.*
            FROM $wpdb->nggpictures AS t 
              INNER JOIN $wpdb->nggallery AS tt ON t.galleryid = tt.gid
            WHERE t.pid IN ($id_list)
              AND t.galleryid = $galleryID
            $exclude_clause
            $order_clause",
            OBJECT_K
            );

    // Build the image objects from the query result
    if ($images) {
      foreach ($images as $key => $image)
        $result[$key] = new nggImage($image);
    }
  }
  return $result;
}

/**
 * Returns all ngg tags of a certain gallery as tag-list (to be used into
 * find_images_for_tags) and as menu (depending on the format)
 */
function nggpowertags_GetGalleryTags($gallery, $format, $separator, $all_word, $exclude) {
  global $nggRewrite;
  
  if (!empty($exclude)) {
    // create an array of trimmed words
    $exclude_arr = array_map('trim', explode(",", $exclude));
  }
  
  // query all ngg tags
  $all_tags = get_terms('ngg_tag');
  foreach ($all_tags as $k => $v) {
    $tag_name = $all_tags[$k]->name;
    if (!isset($exclude_arr) || !in_array($tag_name, $exclude_arr)) {
      $taglist .= $tag_name . ",";
    }
  }
  
  // get all related images to ngg
  $picturelist = nggTags::find_images_for_tags($taglist, 'ASC');
  if (empty($picturelist)) {
    return;
  }
  
  // create rappresentations of the tags for this gallery
  $tags_array = array();
  $taglist = "";
  $a = array();
  
  foreach ($picturelist as $key => $picture) {
    if ($picture->galleryid == $gallery) { // the pic is into our gallery
      // get tags from this pic
      $tmp_picture_tags = wp_get_object_terms($picture->pid,'ngg_tag');
      
      foreach ($tmp_picture_tags as $key => &$tag) {
        if (!isset($tags_array[$tag->term_id])) {
          $tag->link = $nggRewrite->get_permalink(
              array('gallerytag' => $tag->slug)
              );
          $tag->id = $tag->term_id;
          
          $tag_link = '#' != $tag->link ? esc_url($tag->link) : '#';
          $a[$tag->name] = "<a href='$tag_link' class='tag-link-{$tag->id}'>{$tag->name}</a>";
          
          $tags_array[$tag->term_id] = $tag;
          $taglist .= $tag->name . ",";
        }
        
        unset($tag);
      }
    }
  }
  
  ksort($a); // sort by tag name
  // add the "all" link at the beginning
  array_unshift($a, "<a href='" . get_permalink() . "' class='tag-link-all'>$all_word</a>");
  
  switch ($format) :
    case 'list' :
      $tags_format = "<ul class='nggpowertags-list'>\n\t<li>";
      $tags_format .= join("</li>\n\t<li>", $a);
      $tags_format .= "</li>\n</ul>\n";
      break;
    default :
      $tags_format = join($separator, $a);
      break;
  endswitch;
  
  return array(
      $tags_format, // used to print the menu
      $taglist // used to get all pics of this gallery
      );
}

?>